"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const pkgFullName_1 = require('../pkgFullName');
/**
 * Resolves a 'hosted' package hosted on 'github'.
 */
const PARSE_GITHUB_RE = /^github:([^\/]+)\/([^#]+)(#(.+))?$/;
function resolveGithub(pkg, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const getJSON = opts.got.getJSON;
        const spec = parseGithubSpec(pkg);
        spec.ref = yield resolveRef(spec);
        const resPkg = yield resolvePackageJson(spec);
        return {
            name: resPkg.name,
            version: resPkg.version,
            fullname: pkgFullName_1.default({
                name: resPkg.name,
                version: ['github', spec.owner, spec.repo, spec.ref].join(pkgFullName_1.delimiter)
            }),
            dist: {
                tarball: [
                    'https://api.github.com/repos',
                    spec.owner,
                    spec.repo,
                    'tarball',
                    spec.ref
                ].join('/')
            }
        };
        function resolvePackageJson(spec) {
            return __awaiter(this, void 0, void 0, function* () {
                const url = [
                    'https://api.github.com/repos',
                    spec.owner,
                    spec.repo,
                    'contents/package.json?ref=' + spec.ref
                ].join('/');
                const body = yield getJSON(url);
                const content = new Buffer(body.content, 'base64').toString('utf8');
                return JSON.parse(content);
            });
        }
        function resolveRef(spec) {
            return __awaiter(this, void 0, void 0, function* () {
                const url = [
                    'https://api.github.com/repos',
                    spec.owner,
                    spec.repo,
                    'commits',
                    spec.ref
                ].join('/');
                const body = yield getJSON(url);
                return body.sha;
            });
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = resolveGithub;
function parseGithubSpec(pkg) {
    const m = PARSE_GITHUB_RE.exec(pkg.hosted.shortcut);
    if (!m) {
        throw new Error('cannot parse: ' + pkg.hosted.shortcut);
    }
    const owner = m[1];
    const repo = m[2];
    const ref = m[4] || 'HEAD';
    return { owner, repo, ref };
}
//# sourceMappingURL=github.js.map