"use strict";
const npm_1 = require('./npm');
const tarball_1 = require('./tarball');
const github_1 = require('./github');
const local_1 = require('./local');
/**
 * Resolves a package in the NPM registry. Done as part of `install()`.
 *
 *     var npa = require('npm-package-arg')
 *     resolve(npa('rimraf@2'))
 *       .then((res) => {
 *         res.fullname == 'rimraf@2.5.1'
 *         res.dist == {
 *           shasum: '0a1b2c...'
 *           tarball: 'http://...'
 *         }
 *       })
 */
function resolve(pkg, opts) {
    if (pkg.type === 'range' || pkg.type === 'version' || pkg.type === 'tag') {
        return npm_1.default(pkg, opts);
    }
    else if (pkg.type === 'remote') {
        return tarball_1.default(pkg);
    }
    else if (pkg.type === 'hosted' && pkg.hosted.type === 'github') {
        return github_1.default(pkg, opts);
    }
    else if (pkg.type === 'local') {
        return local_1.default(pkg);
    }
    else {
        throw new Error('' + pkg.rawSpec + ': ' + pkg.type + ' packages not supported');
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = resolve;
//# sourceMappingURL=index.js.map